#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        mamewah_ini.py
# Description: read mamewah formatted ini file
# Copyright (c) 2006   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import os

from constants import *

class MameWahIni:
    def __init__(self, ini_filename, ini_type='default', min_version=None):
        """init ini class"""
        self.ini_filename = ini_filename
        self.ini_type = ini_type
        #read file if one had been provided
        if not os.path.exists(ini_filename):
            print "'Error: ini file: [%s] does not exist." % (ini_filename)
        else:
            self.read()
            #check version?
            if min_version:
                all_nums = '0123456789.'
                s = self.lines[0]
                v1 = s[s.find('('):]
                tmp = ''
                try:
                    current_version = tmp.join([c for c in v1 if c in all_nums])
                    if current_version != '':
                        if not self._version_check(min_version, current_version):
                            print "Error: version of ini file: [%s] is: %s.\n  Min version required is: %s" % (
                                ini_filename, current_version, min_version)
                            print "  Get the latest version template from wahcade/config.dist"
                except:
                    pass

    def _version_check(self, v1, v2):
        """returns true if v1 < v2"""
        l = [(int(x) <= int(y)) for x, y in zip(v1.split('.'), v2.split('.'))]
        for test in l:
            if not test:
                return False
        return True

    def _create_dictionary(self):
        """create a dictionary for future lookups"""
        d = {}
        for line in self.lines:
            if not line.startswith('#') and line != '':
                l = line.split(' ', 1)
                #get key and value
                key = str(l[0]).strip()
                if len(l) > 1:
                    value = str(l[1]).strip()
                else:
                    value = ''
                d[key] = value
        #print d
        self.ini_dict = d

    def read(self):
        """read ini file"""
        self.lines = []
        if os.path.isfile(self.ini_filename):
            self.lines = open(self.ini_filename, 'r').readlines()
            self.lines = [s.strip() for s in self.lines]
            self.lines = [s.replace('\t', ' ') for s in self.lines]
        #create lookup dictionary
        self._create_dictionary()

    def write(self):
        """save ini file"""
        #update original lines in ini file from dictionary
        for i, line in enumerate(self.lines):
            option = line[:line.find(' ')]
            if self.ini_dict.has_key(option):
                #build line
                self.lines[i] = '%s%s' % (option.ljust(40), self.ini_dict[option])
        #add correct line terminator
        lines = ['%s\n' % s for s in self.lines]
        #write file
        open(self.ini_filename, 'w').writelines(lines)

    def has_option(self, option):
        """does the option exist"""
        return self.ini_dict.has_key(option)

    def get(self, option):
        """return value for given option"""
        if self.ini_dict.has_key(option):
            value = self.ini_dict[option]
            if value != '':
                test_path = os.path.expanduser(value)
                if os.path.exists(test_path):
                    value = test_path
        else:
            print 'Error: option: [%s] for ini file: [%s] not found' % (option, self.ini_filename)
            value = ''
        #done
        return value

    def getint(self, option):
        """return integer value for given option"""
        #done
        return int(self.get(option))

    def reverse_get(self, value):
        """return possible options for given value"""
        if self.ini_type == 'ctrlr':
            #split keycode values into a list
            d = {}
            for k, v in self.ini_dict.items():
                lv = v.strip(' "').split(' | ')
                d[k] = lv
            #get list of options containing keycode value
            option = [k for k, v in d.items() if value in v]
        else:
            option = [k for k, v in self.ini_dict.items() if value in v]
        #done
        return option

    def set(self, option, value):
        """set option to given value"""
        #if no value, set to empty string
        if value is None:
            value = ''
        self.ini_dict[option] = value



if __name__ == '__main__':
    import sys
    CONFIG_DIR = os.path.join(sys.path[0], 'config.test')

    ini = MameWahIni(os.path.join(CONFIG_DIR, 'wahcade.ini'))
    print 'layout 1=', ini.get('layout')
    ini.set('layout', 'test 1024x768')
    ini.write()
    print 'layout 2=', ini.get('layout')

    #ini = MameWahIni(os.path.join(CONFIG_DIR, 'ini', 'mame.ini'))
    #print 'artwork_1_image_path=', ini.get('artwork_1_image_path')
    #print 'rev: ', ini.reverse_get('c:\\misc')
    #print 'artwork_1_image_path=', ini.get('artwork_1_image_path')
    #ini.set('artwork_1_image_path', 'c:\\misc')
    #print 'artwork_1_image_path=', ini.get('artwork_1_image_path')
    #ini.write()

    #ini = MameWahIni(os.path.join(CONFIG_DIR, 'ctrlr', 'default.ini'), 'ctrlr')
    #print 'EXIT_TO_WINDOWS key=', ini.get('EXIT_TO_WINDOWS')
    #print 'UP_1_GAME       key=', ini.get('UP_1_GAME')
    #print 'DIK_ESCAPE      option=', ini.reverse_get('DIK_ESCAPE')

    #print '*old* DIK_UP          option=', ini.OLDreverse_get('DIK_UP')
    #print 'rev: DIK_UP          option=', ini.reverse_get('DIK_UP')

    #ini = MameWahIni(os.path.expanduser('~/.wahcade/cpviewer.ini'), 'default', '0.10')
    #print 'layout=', ini.get('layout')
    #ini.set('artwork_1_image_path', 'c:\\misc')
    #print 'artwork_1_image_path=', ini.get('artwork_1_image_path')
    #ini.write()
    #print "not exist=",ini.has_option('test-no-exist')
    #print "exist=",ini.has_option('artwork_1_image_path')

